# -*- python -*-
# $RCSfile: activityViewer.py,v $
# $Revision: 1.41 $
# $Author: vrc $
# $Date: 2007-09-26 22:02:24 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

## The Activity Viewer window contains progress bar objects, which are
## defined in progressbarGUI.py.

from ooflib.SWIG.common import switchboard
from ooflib.SWIG.common.IO import progressbar
from ooflib.common import debug
from ooflib.common import guitop
from ooflib.common import mainthread
from ooflib.common import thread_enable
from ooflib.common.IO import oofmenu
from ooflib.common.IO import progressbar_delay
from ooflib.common.IO import threadmanager
from ooflib.common.IO import threadmanagermenu
from ooflib.common.IO.GUI import gtklogger
from ooflib.common.IO.GUI import gtkutils
from ooflib.common.IO.GUI import quit
from ooflib.common.IO.GUI import subWindow
import gobject
import gtk
import sets
import string
import sys
import time

threadManager = threadmanager.threadManager

activityViewer = None

## Has the Activity Viewer been opened?  If the Activity Viewer has
## already been opened, even if it's not open anymore, it's not opened
## automatically (by updatePBdisplay, below).
activityViewerOpened = False

def openActivityViewer():
    debug.mainthreadTest()
    global activityViewer
    global activityViewerOpened
    if activityViewer is None:
        activityViewer = ActivityViewer()
    activityViewer.raise_window()
    activityViewerOpened = True

def sensitize_thread():
    if activityViewer is not None:
        activityViewer.sensitizeButtons()
def sensitize():
    mainthread.runBlock(sensitize_thread)
    
        
class ActivityViewer(subWindow.SubWindow):
    def __init__(self):
        debug.mainthreadTest()
        self.listofgtkbars = []
        # self.pobjs makes sure that Progress objs don't get more than
        # one progress bar.
        self.pobjs = sets.Set()

        self.makeMenus()  # Sets self.menu.
        # SubWindow init sets self.gtk and self.mainbox.
        
        subWindow.SubWindow.__init__(
            self, title="%s Activity Viewer"%subWindow.oofname(), menu=self.menu)

        self.gtk.connect('destroy', self.closeCB)
        self.gtk.set_default_size(400, 300)
        
        # Area at the top containing the editor widget for the line
        self.control_area = gtk.HBox()       # editor widget goes in here
        self.mainbox.pack_start(self.control_area, expand=0, fill=0, padding=2)

        ## create enable-threading button
        self.dismissall = gtkutils.stockButton(gtk.STOCK_CANCEL, "Dismiss All")
        gtklogger.setWidgetName(self.dismissall, "DismissAll")
        gtklogger.connect(self.dismissall, "clicked", self.dismissAll)
        self.control_area.pack_start(self.dismissall, expand=1, fill=0,
                                     padding=2)

        ## create stop-all-threads button
        self.stopall = gtkutils.stockButton(gtk.STOCK_STOP, "Stop All")
        gtklogger.setWidgetName(self.stopall, "StopAll")
        gtklogger.connect(self.stopall, "clicked", self.stopAll)
        self.control_area.pack_start(self.stopall, expand=1, fill=0, padding=2)
                
        # Create a scrolled window to pack the bars into
        self.scrolled_window = gtk.ScrolledWindow()
        self.scrolled_window.set_policy(gtk.POLICY_AUTOMATIC,
                                        gtk.POLICY_AUTOMATIC)
        gtklogger.logScrollBars(self.scrolled_window, "Scroll")
        self.mainbox.pack_start(self.scrolled_window, expand=1, fill=1,
                                padding=2)
        
        ## Create VBox where the progressbars can live happily ever after
        self.bars_play_area = gtk.VBox() # homogeneous=True?
        self.scrolled_window.add_with_viewport(self.bars_play_area)
        
        ## Buttons become sensitized
        self.sensitizeButtons()
        ## show window

        self.initialize() ## add progress bars to Window
        self.gtk.show_all()
        self.sensitizeButtons()
        
    def initialize(self):
        ## Get existing progressbars from the ThreadMonitor and
        ## reconstruct the Window.
        debug.mainthreadTest()
        if not thread_enable.enabled():
            return
        
        for pobj in threadManager.getAllProgressObjects():
            a_newgbar = progressbar.set_ProgressBarRepresentation(pobj)
            self.bars_play_area.pack_start(a_newgbar.gtk,
                                           expand=0, fill=0, padding=3)
            a_newgbar.show()
            self.listofgtkbars.append(a_newgbar)
            self.pobjs.add(pobj)

    def sensitizeButtons(self):
        debug.mainthreadTest()
        threads_in_progress = threadManager.NumThreadsInProgress()
        threads_done = threadManager.NumThreadsDone()
        
        if threads_in_progress >0:
            self.stopall.set_sensitive(1)
        else:
            self.stopall.set_sensitive(0)
            
        if threads_done >0:
            self.dismissall.set_sensitive(1)
        else:
            self.dismissall.set_sensitive(0)

        if not thread_enable.enabled():
            self.stopall.set_sensitive(0)
            self.dismissall.set_sensitive(0)
        
            
    def dismissAll(self, button=None):
        completedbars = [bar for bar in self.listofgtkbars
                         if bar.get_success() != progressbar.INCOMPLETE]
        for bar in completedbars:
            bar.destroy()
        self.sensitizeButtons()

    def stopAll (self, button):
        threadManager.stopAll(button)
        self.sensitizeButtons()

    def dismissgtkbar(self, thisbar):
        debug.mainthreadTest()
        self.listofgtkbars.remove(thisbar)
        self.pobjs.remove(thisbar.progressbar)
        self.sensitizeButtons()
        
    def makeMenus(self): ## Adds quit Button
        self.menu = threadmanagermenu.threadmanagermenu
        # Because we have our own submenu, these are not automatically
        # added in the SubWindow class. 
        self.menu.File.addItem(oofmenu.OOFMenuItem(
            'Close', help="Close this Activity Viewer window.",
            callback=self.close, no_log = 1, gui_only = 1, accel='w',
            threadable = oofmenu.UNTHREADABLE))
        self.menu.File.addItem(oofmenu.OOFMenuItem(
            'Quit', help="TTFN",
            callback=quit.quit, no_log = 1, gui_only = 1, accel='q',
            threadable = oofmenu.UNTHREADABLE))
    
    def closeCB(self, *args):           # GTK callback.
        if activityViewer:
            threadmanagermenu.threadmanagermenu.File.Close()
            

    def close(self, *args):             # Main-thread menu callback.
        debug.mainthreadTest()
        if self.gtk:
            # bar.disconnect() must be called before setting
            # activityViewer to None.
            self.gtk.hide()
            for bar in self.listofgtkbars:
                bar.disconnect()
            self.gtk.destroy()
            self.gtk = None
            global activityViewer
            activityViewer = None

    ## Add a gtkprogressbar to the scrolled window.
    def addGtkbar (self, pbar):
        debug.mainthreadTest()
        # Because the window initializes itself with all existing
        # Progress objects, and because there's a delay between
        # creating a Progress object and opening the window, it's
        # possible that addGtkbar will be called for an object that
        # already has a bar.  self.pobjs keeps track of which objects
        # are already displayed, and checking it prevents them from
        # being displayed twice.  This is a bit of a hack.
        if pbar not in self.pobjs:
            self.pobjs.add(pbar)
            graphicbar = progressbar.set_ProgressBarRepresentation(pbar)
            self.bars_play_area.pack_start(graphicbar.gtk,
                                           expand=0, fill=0, padding=3)
            graphicbar.show()
            self.listofgtkbars.append(graphicbar)
            self.sensitizeButtons()
   
    #####################################################

# Display a new progress bar after a short delay, unless the delay
# time is less than the normal time between progress bar updates, in
# which case display the bar immediately.

def delayed_add_ProgressBar(pbar):
    debug.mainthreadTest()
    if progressbar_delay.get_delay() < progressbar_delay.minimum_time():
        _updatePBdisplay(pbar)          # immediate display
    else:
        gobject.timeout_add(progressbar_delay.get_delay(),
                            _updatePBdisplay, pbar)

# Time-out callback function invoked by delayed_add_ProgressBar.

def _updatePBdisplay(pobj):
    debug.mainthreadTest()
    # Don't do anything unless threads are enabled and the gui has
    # been started.
    if thread_enable.query() and guitop.top():
        # Create just one activity viewer.  If it's been opened and
        # closed already, don't reopen it automatically.  Pre-existing
        # progress bars will be added automatically. 
        if pobj.get_success() != progressbar.INCOMPLETE:
            threadManager.removeProgressBar(pobj)
        else:
            if not activityViewerOpened:
                # The newly created Activity Viewer window
                # automatically calls addGtkbar for every existing
                # Progress object, including the pobj passed as an
                # argument to this routine.
                openActivityViewer()
            else:
                # The Activity Viewer might have been closed, in which
                # case activityViewer is None.
                if activityViewer is not None:
                    activityViewer.addGtkbar(pobj)
    return False

switchboard.requestCallbackMain("new progressbar", delayed_add_ProgressBar)
